@@ -3,6 +3,12 @@ NOMADSRealTimeList <- function(url.type, abbrev = NULL) {
     #Returns a list of model abbreviations for real time models, a short description, and URL for each model offered by the NOMADS server
     #If a specific model abbreviation is requested, the abbreviation is checked against the model list.
     #If a match is found, information is returned about that model; otherwise an error occurs
+    #
+    #
+    #A big shout out to user hrbrmstr on Stack Overflow for providing the table parsing code in this function
+    #http://stackoverflow.com/questions/27592575/dropped-rows-using-readhtmltable-in-r
+    #http://stackoverflow.com/users/1457051/hrbrmstr
+    #
     #INPUTS
     #    URL.TYPE determines which URL to return: one for downloading GRIB files (grib) or one for downloading dods data via DODS (dods)
     #    ABBREV is the model abbreviation that rNOMADS uses to figure out which model you want.
@@ -17,153 +23,87 @@ NOMADSRealTimeList <- function(url.type, abbrev = NULL) {
         stop("URL type must be either \"grib\" or \"dods\"!")
     }
 
-abbrevs <- c(
-    "fnl",
-    "gfs",
-    "gfs_hd",
-    "gfs_2p5",
-    "gens",
-    "gensbc_precip",
-    "gensbc",
-    "gensbc_ndgd",
-    "naefsbc",
-    "naefsbc_ndgd",
-    "ngac_a2d",
-    "ngac_a3d",
-    "ngac_aod",
-    "cfs_flx",
-    "cfs_pgb",
-    "aqm_daily",
-    "aqm_ozone_1hr",
-    "hiresak",
-    "hiresconus",
-    "hiresguam",
-    "hireshi",
-    "hirespr",
-    "nam_ak",
-    "nam",
-    "nam_na",
-    "nam_crb",
-    "nam_pac",
-    "nam_alaskanest",
-    "nam_conusnest",
-    "nam_hawaiinest",
-    "nam_priconest",
-    "akrtma",
-    "rtma2p5",
-    "gurtma",
-    "hirtma",
-    "prrtma",
-    "hrrr_2d",
-    "rap",
-    "rap32",
-    "narre",
-    "sref",
-    "srefbc",
-    "sref_na",
-    "sref_132",
-    "ofs",
-    "rtofs_hires",
-    "rtofs",
-    "seaice",
-    "wave",
-    "glw",
-    "wave_multi",
-    "hurricane_wave",
-    "nfcens",
-    "estofs",
-    "cmcens",
-    "fens",
-    "ncom")
+    base.url <- "http://nomads.ncep.noaa.gov/"
+    trim <- function(x) gsub("^[[:space:]]+|[[:space:]]+$", "", x)
 
-    names <- c(
-    "Final Operational Global Forecast System ",
-    "Global Forecast System 1x1 Degree ",
-    "Global Forecast System 0.5x0.5 Degree ",
-    "Global Forecast System 2.5x2.5 Degree ",
-    "Global Forecast System Ensemble",
-    "Global Forecast System Ensemble Precipitation Bias Corrected ",
-    "Global Forecast System Ensemble Bias Corrected ",
-    "Global Forecast System Ensemble National Digital Guidance Database Bias Corrected ",
-    "North American Ensemble Forecast System Bias Corrected ",
-    "North American Ensemble Forecast System National Digital Guidance Database Bias Corrected ",
-    "NOAA Environmental Modeling System Global Forecast System Aerosol Component 2D",
-    "NOAA Environmental Modeling System Global Forecast System Aerosol Component 3D",
-    "NOAA Environmental Modeling System Global Forecast System Aerosol Optical Depth",
-    "Climate Forecast System Flux Products",
-    "Climate Forecast System 3D Pressure Products",
-    "Air Quality Daily Maximum",
-    "Air Quality Hourly Surface Ozone",
-    "High Res Window Alaska",
-    "High Res Window - Continental United States",
-    "High Res Window - Guam",
-    "High Res Window - Hawaii",
-    "High Res Window - Puerto Rico",
-    "North American Mesoscale 12 km - Alaska",
-    "North American Mesoscale 12 km - Continental United States",
-    "North American Mesoscale 12 km - North America",
-    "North American Mesoscale 12 km - Caribbean and Central America", 
-    "North American Mesoscale 12 km - Pacific",
-    "North American Mesoscale Nest - Alaska",
-    "North American Mesoscale Nest - Continental United States",
-    "North American Mesoscale Nest - Hawaii",
-    "North American Mesoscale Nest - Puerto Rico",
-    "Real-Time Mesoscale Analysis - Alaska",
-    "Real Time Mesoscale Analysis - Continental United States 2.5 km Resolution",
-    "Real Time Mesoscale Analysis - Guam",
-    "Real Time Mesoscale Analysis - Hawaii",
-    "Real Time Mesoscale Analysis - Puerto Rico",
-    "High Resolution Rapid Refresh",
-    "Rapid Refresh Weather Prediction System",
-    "Rapid Refresh Weather Prediction System - 32 km Resolution",
-    "North American Rapid Refresh Ensemble",
-    "Short Range Ensemble Forecast - Continental United States 40 km",
-    "Short Range Ensemble Forecast - Continental United States 40 km Bias Corrected",
-    "Short Range Ensemble Forecast - Continental United States 32 km",
-    "Short Range Ensemble Forecast - Continental United States 16 km",
-    "Real Time Ocean Forecast System - Atlantic",
-    "Real Time Ocean Forecast System - Atlantic High Resolution",
-    "Real Time Ocean Forecast System - Global",
-    "Sea Ice",
-    "Operational Ocean Wave Predictions",
-    "Operational Ocean Wave Predictions - Great Lakes",
-    "Multi-grid Wave",
-    "Hurricane Wave",
-    "Combined Wave Ensemble",
-    "Extratropical Surge and Tide Operational Forecast System",
-    "Canadian Meterological Center Global Ensemble",
-    "Fleet Numerical Meteorology and Oceanography Ensemble Forecast System",
-    "U.S. Navy Operational Global Ocean Model") 
+    # neither rvest::html nor rvest::html_session liked it, hence using XML::htmlParse
+    doc <- XML::htmlParse(base.url)
 
+    ds <- doc %>% html_nodes(xpath="//table/descendant::th[@class='nomads'][1]/../../
+                                            descendant::td[contains(., 'http')]/
+                                            preceding-sibling::td[3]")
+    data.set <- ds %>% html_text() %>% trim()
 
-    if(!is.null(abbrev)) {
-        i <- which(abbrevs == abbrev)
-        if(length(i) == 0) {
-            stop(paste("The model you searched for:\"", abbrev, "\"is not included in rNOMADS.  Sorry!"))
-        } else {
-            if(url.type == "grib") {
-                 url <- paste0("http://nomads.ncep.noaa.gov/cgi-bin/filter_", abbrev, ".pl")
-            } else {
-                url <- paste0("http://nomads.ncep.noaa.gov:9090/dods/", abbrev, "/")
-            }
-        }
-    } else {
-        if(url.type == "grib") {
-           url <- paste0("http://nomads.ncep.noaa.gov/cgi-bin/filter_", abbrevs, ".pl") 
-        } else {
-           url <- paste0("http://nomads.ncep.noaa.gov:9090/dods/", abbrevs, "/")
-        }
+#    html_text() %>% trim()
+
+    grib.filter <- doc %>% html_nodes(xpath="//table/descendant::th[@class='nomads'][1]/../../
+                                  descendant::td[contains(., 'http')]/preceding-sibling::td[1]") %>%
+   sapply(function(x) {
+     ifelse(x %>% xpathApply("boolean(./a)"),
+           x %>% html_node("a") %>% html_attr("href"),
+           NA)
+    })
+
+   http.link <- doc %>% html_nodes("a[href^='/pub/data/']") %>% html_attr("href")
+
+   gds.alt <- doc %>% html_nodes(xpath="//table/descendant::th[@class='nomads'][1]/../../
+                              descendant::td[contains(., 'http')]/following-sibling::td[1]") %>%
+   sapply(function(x) {
+    ifelse(x %>% xpathApply("boolean(./a)"),
+           x %>% html_node("a") %>% html_attr("href"),
+           NA)
+   })
  
+   grib.abbrevs <- stringr::str_replace(stringr::str_replace(basename(grib.filter), "filter_", ""), ".pl", "")
+   dods.abbrevs <- basename(gds.alt)
+   dods.base.url <- "http://nomads.ncep.noaa.gov:9090/dods/"
+   if(is.null(abbrev)) {
+       if(url.type == "grib") {
+          good.abbrevs <- which(!is.na(grib.abbrevs))
+          model.list <- list(abbrev = grib.abbrevs[good.abbrevs], 
+               name = data.set[good.abbrevs], 
+               url = paste0(base.url, 
+               grib.filter[good.abbrevs]))
+       } else {
+          good.abbrevs <- which(!is.na(dods.abbrevs))
+          model.list <- list(abbrev = dods.abbrevs[good.abbrevs], 
+              name = data.set[good.abbrevs], 
+              url = paste0(dods.base.url, 
+              basename(gds.alt[good.abbrevs]), "/"))
+       }
+  } else {
+      if(url.type == "grib") {
+           abbrev.ind <- which(abbrev == grib.abbrevs)
+           if(length(abbrev.ind) > 0) {
+               model.list <- list(abbrev = grib.abbrevs[abbrev.ind], 
+                   name = data.set[abbrev.ind], 
+                   url = paste0(base.url, grib.filter[abbrev.ind]))
+           } else {
+                stop(paste0("The model you searched for: \"", 
+                    abbrev, 
+                    "\" is not included in NOMADS real time ", 
+                    url.type,
+                     " model products.  Sorry!"))    
+           }
+      } else {
+           abbrev.ind <- which(abbrev == dods.abbrevs)
+           if(length(abbrev.ind) > 0) {
+               model.list <- list(abbrev = dods.abbrevs[abbrev.ind], 
+                   name = data.set[abbrev.ind], 
+                   url = paste0(dods.base.url, basename(gds.alt[abbrev.ind]), "/"))
+           } else {
+                stop(paste0("The model you searched for: \"", 
+                    abbrev, 
+                    "\" is not included in NOMADS real time ", 
+                    url.type, 
+                     " model products.  Sorry!"))
+           }
+       }
    }
-
-    #Handle known exceptions
-    if(url.type == "dods") {
-        url <- stringr::str_replace(url, "hrrr_2d", "hrrr")
-    }
-
-    return(list(abbrevs = abbrevs, names = names, url = url))
+   
+   return(model.list)
 }
-
+   
 NOMADSArchiveList <- function(url.type, abbrev = NULL) {
     #Returns a list of model abbreviations for archived models, a short description, and URL for each model offered by the NOMADS server
     #If a specific model abbreviation is requested, the abbreviation is checked against the model list.
